-------------------------------------------------------------------------------
-- video_to_ram_0_wrapper.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity video_to_ram_0_wrapper is
  port (
    i_sys_rst : in std_logic;
    i_clk_100_bus : in std_logic;
    i_LLC_CLOCK : in std_logic;
    i_YCrCb : in std_logic_vector(9 downto 2);
    o_M_abort : out std_logic;
    o_M_ABus : out std_logic_vector(0 to 31);
    o_M_BE : out std_logic_vector(0 to 3);
    o_M_busLock : out std_logic;
    o_M_TAttribute : out std_logic_vector(0 to 15);
    o_M_lockErr : out std_logic;
    o_M_mSize : out std_logic_vector(0 to 1);
    o_M_priority : out std_logic_vector(0 to 1);
    o_M_rdBurst : out std_logic;
    o_M_request : out std_logic;
    o_M_RNW : out std_logic;
    o_M_size : out std_logic_vector(0 to 3);
    o_M_type : out std_logic_vector(0 to 2);
    o_M_wrBurst : out std_logic;
    o_M_wrDBus : out std_logic_vector(0 to 31);
    i_PLB_MAddrAck : in std_logic;
    i_PLB_MBusy : in std_logic;
    i_PLB_MRdErr : in std_logic;
    i_PLB_MWrErr : in std_logic;
    i_PLB_MRdBTerm : in std_logic;
    i_PLB_MRdDAck : in std_logic;
    i_PLB_MRdDBus : in std_logic_vector(0 to 31);
    i_PLB_MRdWdAddr : in std_logic_vector(0 to 3);
    i_PLB_MRearbitrate : in std_logic;
    i_PLB_MSSize : in std_logic_vector(0 to 1);
    i_PLB_MWrBTerm : in std_logic;
    i_PLB_MWrDAck : in std_logic;
    i_PLB_MTimeout : in std_logic;
    i_PLB_PAValid : in std_logic;
    i_Sl_addrAck : in std_logic_vector(0 to 0);
    i_Sl_wrDAck : in std_logic_vector(0 to 0);
    i_Sl_wrComp : in std_logic_vector(0 to 0);
    i_MPMC_Done_Init : in std_logic;
    i_DBG_new_line : in std_logic;
    o_DBG_fsm_cs : out std_logic_vector(3 downto 0);
    o_DBG_general_purpose : out std_logic_vector(3 downto 0)
  );

  attribute x_core_info : STRING;
  attribute x_core_info of video_to_ram_0_wrapper : entity is "video_to_ram_v1_00_a";

end video_to_ram_0_wrapper;

architecture STRUCTURE of video_to_ram_0_wrapper is

  component video_to_ram is
    generic (
      C_PLBV46_NUM_SLAVES : INTEGER;
      C_PLBV46_DWIDTH : INTEGER;
      C_VIDEO_RAM_BASEADDR : INTEGER;
      C_BYTES_PER_LINE : INTEGER
    );
    port (
      i_sys_rst : in std_logic;
      i_clk_100_bus : in std_logic;
      i_LLC_CLOCK : in std_logic;
      i_YCrCb : in std_logic_vector(9 downto 2);
      o_M_abort : out std_logic;
      o_M_ABus : out std_logic_vector(0 to 31);
      o_M_BE : out std_logic_vector(0 to ((C_PLBV46_DWIDTH/8)-1));
      o_M_busLock : out std_logic;
      o_M_TAttribute : out std_logic_vector(0 to 15);
      o_M_lockErr : out std_logic;
      o_M_mSize : out std_logic_vector(0 to 1);
      o_M_priority : out std_logic_vector(0 to 1);
      o_M_rdBurst : out std_logic;
      o_M_request : out std_logic;
      o_M_RNW : out std_logic;
      o_M_size : out std_logic_vector(0 to 3);
      o_M_type : out std_logic_vector(0 to 2);
      o_M_wrBurst : out std_logic;
      o_M_wrDBus : out std_logic_vector(0 to 31);
      i_PLB_MAddrAck : in std_logic;
      i_PLB_MBusy : in std_logic;
      i_PLB_MRdErr : in std_logic;
      i_PLB_MWrErr : in std_logic;
      i_PLB_MRdBTerm : in std_logic;
      i_PLB_MRdDAck : in std_logic;
      i_PLB_MRdDBus : in std_logic_vector(0 to 31);
      i_PLB_MRdWdAddr : in std_logic_vector(0 to 3);
      i_PLB_MRearbitrate : in std_logic;
      i_PLB_MSSize : in std_logic_vector(0 to 1);
      i_PLB_MWrBTerm : in std_logic;
      i_PLB_MWrDAck : in std_logic;
      i_PLB_MTimeout : in std_logic;
      i_PLB_PAValid : in std_logic;
      i_Sl_addrAck : in std_logic_vector(0 to (C_PLBV46_NUM_SLAVES-1));
      i_Sl_wrDAck : in std_logic_vector(0 to (C_PLBV46_NUM_SLAVES-1));
      i_Sl_wrComp : in std_logic_vector(0 to (C_PLBV46_NUM_SLAVES-1));
      i_MPMC_Done_Init : in std_logic;
      i_DBG_new_line : in std_logic;
      o_DBG_fsm_cs : out std_logic_vector(3 downto 0);
      o_DBG_general_purpose : out std_logic_vector(3 downto 0)
    );
  end component;

begin

  video_to_ram_0 : video_to_ram
    generic map (
      C_PLBV46_NUM_SLAVES => 1,
      C_PLBV46_DWIDTH => 32,
      C_VIDEO_RAM_BASEADDR => 1073741824,
      C_BYTES_PER_LINE => 4096
    )
    port map (
      i_sys_rst => i_sys_rst,
      i_clk_100_bus => i_clk_100_bus,
      i_LLC_CLOCK => i_LLC_CLOCK,
      i_YCrCb => i_YCrCb,
      o_M_abort => o_M_abort,
      o_M_ABus => o_M_ABus,
      o_M_BE => o_M_BE,
      o_M_busLock => o_M_busLock,
      o_M_TAttribute => o_M_TAttribute,
      o_M_lockErr => o_M_lockErr,
      o_M_mSize => o_M_mSize,
      o_M_priority => o_M_priority,
      o_M_rdBurst => o_M_rdBurst,
      o_M_request => o_M_request,
      o_M_RNW => o_M_RNW,
      o_M_size => o_M_size,
      o_M_type => o_M_type,
      o_M_wrBurst => o_M_wrBurst,
      o_M_wrDBus => o_M_wrDBus,
      i_PLB_MAddrAck => i_PLB_MAddrAck,
      i_PLB_MBusy => i_PLB_MBusy,
      i_PLB_MRdErr => i_PLB_MRdErr,
      i_PLB_MWrErr => i_PLB_MWrErr,
      i_PLB_MRdBTerm => i_PLB_MRdBTerm,
      i_PLB_MRdDAck => i_PLB_MRdDAck,
      i_PLB_MRdDBus => i_PLB_MRdDBus,
      i_PLB_MRdWdAddr => i_PLB_MRdWdAddr,
      i_PLB_MRearbitrate => i_PLB_MRearbitrate,
      i_PLB_MSSize => i_PLB_MSSize,
      i_PLB_MWrBTerm => i_PLB_MWrBTerm,
      i_PLB_MWrDAck => i_PLB_MWrDAck,
      i_PLB_MTimeout => i_PLB_MTimeout,
      i_PLB_PAValid => i_PLB_PAValid,
      i_Sl_addrAck => i_Sl_addrAck,
      i_Sl_wrDAck => i_Sl_wrDAck,
      i_Sl_wrComp => i_Sl_wrComp,
      i_MPMC_Done_Init => i_MPMC_Done_Init,
      i_DBG_new_line => i_DBG_new_line,
      o_DBG_fsm_cs => o_DBG_fsm_cs,
      o_DBG_general_purpose => o_DBG_general_purpose
    );

end architecture STRUCTURE;

