//----------------------------------------------------------------------------
// user_logic.vhd - module
//----------------------------------------------------------------------------
//
// ***************************************************************************
// ** Copyright (c) 1995-2008 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** Xilinx, Inc.                                                          **
// ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
// ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
// ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
// ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
// ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
// ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
// ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
// ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
// ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
// ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
// ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
// ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
// ** FOR A PARTICULAR PURPOSE.                                             **
// **                                                                       **
// ***************************************************************************
//
//----------------------------------------------------------------------------
// Filename:          user_logic.vhd
// Version:           1.00.a
// Description:       User logic module.
// Date:              Sat Mar 19 17:00:14 2011 (by Create and Import Peripheral Wizard)
// Verilog Standard:  Verilog-2001
//----------------------------------------------------------------------------
// Naming Conventions:
//   active low signals:                    "*_n"
//   clock signals:                         "clk", "clk_div#", "clk_#x"
//   reset signals:                         "rst", "rst_n"
//   generics:                              "C_*"
//   user defined types:                    "*_TYPE"
//   state machine next state:              "*_ns"
//   state machine current state:           "*_cs"
//   combinatorial signals:                 "*_com"
//   pipelined or register delay signals:   "*_d#"
//   counter signals:                       "*cnt*"
//   clock enable signals:                  "*_ce"
//   internal version of output port:       "*_i"
//   device pins:                           "*_pin"
//   ports:                                 "- Names begin with Uppercase"
//   processes:                             "*_PROCESS"
//   component instantiations:              "<ENTITY_>I_<#|FUNC>"
//----------------------------------------------------------------------------

module user_logic
(
  // -- ADD USER PORTS BELOW THIS LINE ---------------
  // --USER ports added here  
  i_MPMC_Done_Init,  
  // -- ADD USER PORTS ABOVE THIS LINE ---------------

  // -- DO NOT EDIT BELOW THIS LINE ------------------
  // -- Bus protocol ports, do not add to or delete 
  Bus2IP_Clk,                     // Bus to IP clock
  Bus2IP_Reset,                   // Bus to IP reset
  Bus2IP_Addr,                    // Bus to IP address bus
  Bus2IP_Data,                    // Bus to IP data bus
  Bus2IP_BE,                      // Bus to IP byte enables
  Bus2IP_RdCE,                    // Bus to IP read chip enable
  Bus2IP_WrCE,                    // Bus to IP write chip enable
  IP2Bus_Data,                    // IP to Bus data bus
  IP2Bus_RdAck,                   // IP to Bus read transfer acknowledgement
  IP2Bus_WrAck,                   // IP to Bus write transfer acknowledgement
  IP2Bus_Error,                   // IP to Bus error response
  
  //PLB master signals  
  IP2Bus_MstRd_Req,               // IP to Bus master read request
  IP2Bus_MstWr_Req,               // IP to Bus master write request
  IP2Bus_Mst_Addr,                // IP to Bus master address bus
  IP2Bus_Mst_BE,                  // IP to Bus master byte enables
  IP2Bus_Mst_Lock,                // IP to Bus master lock
  IP2Bus_Mst_Reset,               // IP to Bus master reset
  Bus2IP_Mst_CmdAck,              // Bus to IP master command acknowledgement
  Bus2IP_Mst_Cmplt,               // Bus to IP master transfer completion
  Bus2IP_Mst_Error,               // Bus to IP master error response
  Bus2IP_Mst_Rearbitrate,         // Bus to IP master re-arbitrate
  Bus2IP_Mst_Cmd_Timeout,         // Bus to IP master command timeout
  Bus2IP_MstRd_d,                 // Bus to IP master read data bus
  Bus2IP_MstRd_src_rdy_n,         // Bus to IP master read source ready
  IP2Bus_MstWr_d,                 // IP to Bus master write data bus
  Bus2IP_MstWr_dst_rdy_n          // Bus to IP master write destination ready
  // -- DO NOT EDIT ABOVE THIS LINE ------------------
); // user_logic

// -- ADD USER PARAMETERS BELOW THIS LINE ------------
// --USER parameters added here 
parameter C_PLBV46_NUM_SLAVES 	        = 1;
parameter C_PLBV46_DWIDTH     	        = 32;
parameter C_VIDEO_RAM_BASEADDR			  = 32'h40000000;
parameter C_VIDEO_OUT_BASEADDR           = 32'h43000000;
parameter C_BYTES_PER_LINE				     = 4096;

// FSM States
localparam						         C_STATE_BITS		        = 4;
localparam	[C_STATE_BITS-1:0]	   INIT  		              = 0,
									         START_OF_FRAME				  = 1,
												REQUEST			  			  = 2,
												WAITFORDATA     			  = 3,
												PROCESSDATA				     = 4,
												WRITE_REQUEST             = 5,
												WRITE                     = 6,
												CALCULATE                 = 7;
// -- ADD USER PARAMETERS ABOVE THIS LINE ------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol parameters, do not add to or delete
parameter C_SLV_DWIDTH                   = 32;
parameter C_MST_AWIDTH                   = 32;
parameter C_MST_DWIDTH                   = 32;
parameter C_NUM_REG                      = 28;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

// -- ADD USER PORTS BELOW THIS LINE -----------------
// --USER ports added here 
input 												i_MPMC_Done_Init;
// -- ADD USER PORTS ABOVE THIS LINE -----------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol ports, do not add to or delete
input                                     Bus2IP_Clk;
input                                     Bus2IP_Reset;
input      [0 : 31]                       Bus2IP_Addr;
input      [0 : C_SLV_DWIDTH-1]           Bus2IP_Data;
input      [0 : C_SLV_DWIDTH/8-1]         Bus2IP_BE;
input      [0 : C_NUM_REG-1]              Bus2IP_RdCE;
input      [0 : C_NUM_REG-1]              Bus2IP_WrCE;
output     [0 : C_SLV_DWIDTH-1]           IP2Bus_Data;
output                                    IP2Bus_RdAck;
output                                    IP2Bus_WrAck;
output                                    IP2Bus_Error;

//PLB master signals
output                                    IP2Bus_MstRd_Req;
output                                    IP2Bus_MstWr_Req;
output     [0 : C_MST_AWIDTH-1]           IP2Bus_Mst_Addr;
output     [0 : C_MST_DWIDTH/8-1]         IP2Bus_Mst_BE;
output                                    IP2Bus_Mst_Lock;
output                                    IP2Bus_Mst_Reset;
input                                     Bus2IP_Mst_CmdAck;
input                                     Bus2IP_Mst_Cmplt;
input                                     Bus2IP_Mst_Error;
input                                     Bus2IP_Mst_Rearbitrate;
input                                     Bus2IP_Mst_Cmd_Timeout;
input      [0 : C_MST_DWIDTH-1]           Bus2IP_MstRd_d;
input                                     Bus2IP_MstRd_src_rdy_n;
output     [0 : C_MST_DWIDTH-1]           IP2Bus_MstWr_d;
input                                     Bus2IP_MstWr_dst_rdy_n;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

//----------------------------------------------------------------------------
// Implementation
//----------------------------------------------------------------------------
  
  // Declaring Bus Protocol nets as Signals
  reg 								IP2Bus_MstRd_Req;
  wire 								IP2Bus_Mst_Lock;
  wire 								IP2Bus_Mst_Reset;
  reg  								IP2Bus_MstWr_Req;
  wire [0 : C_MST_DWIDTH/8-1] IP2Bus_Mst_BE;
  wire 								IP2Bus_Error; 
  reg  [0 : 31]               IP2Bus_Mst_Addr; 
  reg  [0 : 31] 					IP2Bus_MstWr_d;  
  
  // --USER nets declarations added here, as needed for user logic       
  // Signals to get the right color values
  wire   [0:7]						   		  w_red_value;
  wire	[0:7]									  w_green_value;
  wire	[0:7]									  w_blue_value;
  // Local registers in the user logic
  reg    [0:31] 	 							  frame_x;
  reg    [0:31] 	 							  frame_y;
  reg    [0:31] 	 							  pixAddr;
  reg    [0:31]                          pixIndex;
  reg    [0:31] 								  dataWord;
  reg    [0:31]                          frameTime;
  // State Machine
  reg		[C_STATE_BITS-1:0]				  r_current_state;
  reg		[0:31]				  				  r_pixel_write;
  reg		[0:31]				  				  r_write_addr;            
  
  reg												  r_start; 
  reg                                 r_ready_rec_1; 
  reg                                 r_ready_rec_2;
  reg                                 r_ready_rec_3;
  reg                                 r_ready_rec_4;
  reg												  r_done_1;
  reg												  r_done_2;
  reg												  r_done_3;
  reg												  r_done_4;
  
  wire   										  d_start;
  wire   										  d_ready_1;
  wire 	[0:31]								  d_dividend_1;  
  wire   [0:31]								  d_divider_1;
  wire   [0:31]								  d_quotient_1;
  wire                    d_ready_rec_1; 
  
  wire   										  d_ready_2;
  wire 	[0:31]								  d_dividend_2;  
  wire   [0:31]								  d_divider_2;
  wire   [0:31]								  d_quotient_2;
  wire                    d_ready_rec_2;
  
  wire   										  d_ready_3;
  wire 	[0:31]								  d_dividend_3;  
  wire   [0:31]								  d_divider_3;
  wire   [0:31]								  d_quotient_3;
  wire                    d_ready_rec_3;
  
  wire   										  d_ready_4;
  wire 	[0:31]								  d_dividend_4;  
  wire   [0:31]								  d_divider_4;
  wire   [0:31]								  d_quotient_4;
  wire                    d_ready_rec_4;
  
  reg        [0 : C_SLV_DWIDTH-1]           r_player1_point1_x;
  reg        [0 : C_SLV_DWIDTH-1]           r_player1_point1_y;
  reg        [0 : C_SLV_DWIDTH-1]           r_player1_point2_x;
  reg        [0 : C_SLV_DWIDTH-1]           r_player1_point2_y;
  reg        [0 : C_SLV_DWIDTH-1]           r_player1_point1_count;
  reg        [0 : C_SLV_DWIDTH-1]           r_player1_point2_count;
  reg        [0 : C_SLV_DWIDTH-1]           r_player2_point1_x;
  reg        [0 : C_SLV_DWIDTH-1]           r_player2_point1_y;
  reg        [0 : C_SLV_DWIDTH-1]           r_player2_point2_x;
  reg        [0 : C_SLV_DWIDTH-1]           r_player2_point2_y;
  reg        [0 : C_SLV_DWIDTH-1]           r_player2_point1_count;
  reg        [0 : C_SLV_DWIDTH-1]           r_player2_point2_count;

  reg        [0 : C_SLV_DWIDTH-1]           prev_player1_point1_x;
  reg        [0 : C_SLV_DWIDTH-1]           prev_player1_point1_y;
  reg        [0 : C_SLV_DWIDTH-1]           prev_player2_point1_x;
  reg        [0 : C_SLV_DWIDTH-1]           prev_player2_point1_y;
  reg        [0 : C_SLV_DWIDTH-1]           prev_player1_point1_count;
  reg        [0 : C_SLV_DWIDTH-1]           prev_player2_point1_count;

  // Nets for user logic slave model s/w accessible register example
  reg        [0 : C_SLV_DWIDTH-1]           r_frame_base_addr;
  reg        [0 : C_SLV_DWIDTH-1]           r_run;
  reg        [0 : C_SLV_DWIDTH-1]           r_rgb_point1_low;
  reg        [0 : C_SLV_DWIDTH-1]           r_rgb_point1_high;
  reg        [0 : C_SLV_DWIDTH-1]           r_rgb_point2_low;
  reg        [0 : C_SLV_DWIDTH-1]           r_rgb_point2_high;
  reg        [0 : C_SLV_DWIDTH-1]           r_rgb_point3_low;
  reg        [0 : C_SLV_DWIDTH-1]           r_rgb_point3_high;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg8;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg9;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg10;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg11;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg12;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg13;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg14;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg15;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg16;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg17;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg18;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg19;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg_20;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg_21;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg_22;
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg_23;
  wire       [0 : 23]                       slv_reg_write_sel;
  wire       [0 : 23]                       slv_reg_read_sel;
  reg        [0 : C_SLV_DWIDTH-1]           slv_ip2bus_data;
  wire                                      slv_read_ack;
  wire                                      slv_write_ack;
  integer                                   byte_index, bit_index;

  // --USER logic implementation added here

  initial
   begin
     r_rgb_point1_high <= 32'b11111111010110100101101000000000;
     r_rgb_point1_low <= 32'b10101010000000000000000000000000;
   end
   
  assign
    slv_reg_write_sel = Bus2IP_WrCE[0:23],
    slv_reg_read_sel  = Bus2IP_RdCE[0:23],
    slv_write_ack     = Bus2IP_WrCE[0] || Bus2IP_WrCE[1] || Bus2IP_WrCE[2] || Bus2IP_WrCE[3] || Bus2IP_WrCE[4] || Bus2IP_WrCE[5] || Bus2IP_WrCE[6] || Bus2IP_WrCE[7] || Bus2IP_WrCE[8] || Bus2IP_WrCE[9] || Bus2IP_WrCE[10] || Bus2IP_WrCE[11] || Bus2IP_WrCE[12] || Bus2IP_WrCE[13] || Bus2IP_WrCE[14] || Bus2IP_WrCE[15] || Bus2IP_WrCE[16] || Bus2IP_WrCE[17] || Bus2IP_WrCE[18] || Bus2IP_WrCE[19] || Bus2IP_WrCE[20] || Bus2IP_WrCE[21] || Bus2IP_WrCE[22] || Bus2IP_WrCE[23],
    slv_read_ack      = Bus2IP_RdCE[0] || Bus2IP_RdCE[1] || Bus2IP_RdCE[2] || Bus2IP_RdCE[3] || Bus2IP_RdCE[4] || Bus2IP_RdCE[5] || Bus2IP_RdCE[6] || Bus2IP_RdCE[7] || Bus2IP_RdCE[8] || Bus2IP_RdCE[9] || Bus2IP_RdCE[10] || Bus2IP_RdCE[11] || Bus2IP_RdCE[12] || Bus2IP_RdCE[13] || Bus2IP_RdCE[14] || Bus2IP_RdCE[15] || Bus2IP_RdCE[16] || Bus2IP_RdCE[17] || Bus2IP_RdCE[18] || Bus2IP_RdCE[19] || Bus2IP_RdCE[20] || Bus2IP_RdCE[21] || Bus2IP_RdCE[22] || Bus2IP_RdCE[23];
  
  assign w_red_value   = slv_reg_20[8:15];
  assign w_green_value = slv_reg_20[16:23];
  assign w_blue_value  = slv_reg_20[24:31];
      	
  assign d_dividend_1 = r_player1_point1_x;  
  assign d_divider_1  = r_player1_point1_count;
  assign d_ready_rec_1= r_ready_rec_1;  
  
  assign d_dividend_2 = r_player1_point1_y;
  assign d_divider_2  = r_player1_point1_count;  
  assign d_ready_rec_2= r_ready_rec_2;
  
  assign d_dividend_3 = r_player2_point1_x;  
  assign d_divider_3  = r_player2_point1_count;  
  assign d_ready_rec_3= r_ready_rec_3;
  
  assign d_dividend_4 = r_player2_point1_y;
  assign d_divider_4  = r_player2_point1_count;
  assign d_ready_rec_4= r_ready_rec_4;
  
  assign d_start = r_start;
  
  assign 	  
     IP2Bus_Mst_Lock = 1'b0,
	  IP2Bus_Mst_Reset = Bus2IP_Reset,	  
	  IP2Bus_Mst_BE = 4'b1111;	  
//	  pixAddr = frame_y[12:31] & frame_x[22:31] & 2'b00,     
//	  IP2Bus_Mst_Addr = C_VIDEO_RAM_BASEADDR + pixAddr;	     
    
//  always @( * )
//    begin
//	   slv_reg_21 <= r_current_state;
//		slv_reg_22 <= pixAddr;				
//	 end

  always @( posedge Bus2IP_Clk)
    begin
	   if( Bus2IP_Reset == 1'b1)
		  begin
			 slv_reg_21 <= 0;
			 slv_reg_22 <= 0;
		    slv_reg_23 <= 0;		 
		  end		
		slv_reg_21 <= r_current_state;
		slv_reg_22 <= pixAddr;
		slv_reg_23[31] <= Bus2IP_Mst_CmdAck;
		slv_reg_23[30] <= Bus2IP_Mst_Cmplt;
		if( r_current_state == WRITE )
		  slv_reg_23[29] <= 1;
    end
	 
  task reset_variables;	   
	 begin			 			
          // Reset finger point locations of players to 0			 			 
			 frame_x <= 0;
			 frame_y <= 0;
			 pixAddr <= 0;
			 
          r_player1_point1_x <= 0;
          r_player1_point1_y <= 0;
          r_player1_point2_x <= 0;
          r_player1_point2_y <= 0;
          r_player1_point1_count <= 0;
          r_player1_point2_count <= 0;
			 
          r_player2_point1_x <= 0;
          r_player2_point1_y <= 0;
          r_player2_point2_x <= 0;
          r_player2_point2_y <= 0;
          r_player2_point1_count <= 0;
          r_player2_point2_count <= 0;           			
    end
  endtask    	  
  
  always @( posedge Bus2IP_Clk )
    begin
      if ( Bus2IP_Reset == 1 )
        begin
		    IP2Bus_MstWr_Req <= 0;
		    IP2Bus_MstRd_Req <= 0;		  
		    r_current_state <= INIT;
			 frame_x <= 0;
			 frame_y <= 0;
			 dataWord <= 0;			      
          prev_player1_point1_x <= 0;
          prev_player1_point1_y <= 0;
          prev_player2_point1_x <= 0;
          prev_player2_point1_y <= 0;
          prev_player1_point1_count <= 0;
          prev_player2_point1_count <= 0;			 
			 
		    slv_reg8 <= 0;
          slv_reg9 <= 0;
          slv_reg10 <= 0;
          slv_reg11 <= 0;
          slv_reg12 <= 0;
          slv_reg13 <= 0;
          slv_reg14 <= 0;
          slv_reg15 <= 0;
          slv_reg16 <= 0;
          slv_reg17 <= 0;
          slv_reg18 <= 0;
          slv_reg19 <= 0;
          slv_reg_20 <= 0;
          reset_variables;
        end
      else begin		  
        case(r_current_state)
			  INIT: begin
			    if ( i_MPMC_Done_Init == 1'b1 ) begin
				   r_current_state <= START_OF_FRAME;
			    end
			    else begin
				   r_current_state <= INIT;
			    end				 
			  end
				 
			  START_OF_FRAME: begin
				 reset_variables;
				 r_current_state <= REQUEST;					 
			  end
			  
			  REQUEST: begin			    				 				 
				 IP2Bus_MstRd_Req <= 1'b1;				 				 
				 slv_reg_20 <= Bus2IP_MstRd_d;
             IP2Bus_Mst_Addr <= C_VIDEO_RAM_BASEADDR + pixAddr;				              			
					
				 if( Bus2IP_Mst_CmdAck == 1'b1 )
				   begin
					  IP2Bus_MstRd_Req <= 1'b0;
					  r_current_state <= WAITFORDATA;					  
				   end
					
				 if( Bus2IP_Mst_Cmplt == 1'b1 )
				   begin
					  r_current_state <= PROCESSDATA;          					  
				   end
			  end	
			  
			  WAITFORDATA: begin			  
			    slv_reg_20 <= Bus2IP_MstRd_d;
				 IP2Bus_Mst_Addr <= C_VIDEO_RAM_BASEADDR + pixAddr;				 			
				 
				 if (Bus2IP_Mst_Cmplt == 1'b1) 
				   begin				   
					  r_current_state <= PROCESSDATA;					  
				   end
				 else
					r_current_state <= WAITFORDATA;          				 				
			  end	 		  		  		  		 		  
			  
			  PROCESSDATA: begin			    			      
             r_pixel_write <= 0;
             r_write_addr <= C_VIDEO_OUT_BASEADDR + pixAddr;             			
				 
				 if( w_red_value > r_rgb_point1_low[0:7] && w_red_value < r_rgb_point1_high[0:7] &&
					  w_green_value > r_rgb_point1_low[8:15] && w_green_value < r_rgb_point1_high[8:15] &&
					  w_blue_value > r_rgb_point1_low[16:23] && w_blue_value < r_rgb_point1_high[16:23] ) begin
					  
					r_pixel_write <= slv_reg_20;
					
				   r_player1_point1_x <= r_player1_point1_x + frame_x;
					r_player1_point1_y <= r_player1_point1_y + frame_y;
					r_player1_point1_count <= r_player1_point1_count + 1;						 
				 end
					
				 if( w_red_value > r_rgb_point2_low[0:7] && w_red_value < r_rgb_point2_high[0:7] &&
				 	  w_green_value > r_rgb_point2_low[8:15] && w_green_value < r_rgb_point2_high[8:15] &&
					  w_blue_value > r_rgb_point2_low[16:23] && w_blue_value < r_rgb_point2_high[16:23] ) begin
					
					r_pixel_write <= slv_reg_20;
					
               r_player2_point1_x <= r_player2_point1_x + frame_x;
					r_player2_point1_y <= r_player2_point1_y + frame_y;
					r_player2_point1_count <= r_player2_point1_count + 1;																								
				 end
				 
				 if( frame_x >= 639 ) begin
				  	frame_x <= 0;
					pixAddr <= pixAddr + 1540;
					  if( frame_y >= 479) begin
					    frame_y <= 0;						 						 			 			          												 						                    																	
						 pixAddr <= 0;
						 
						 r_start <= 0;
						 r_current_state <= CALCULATE;
					  end
					  else begin					    
					    r_current_state <= WRITE_REQUEST;
					    frame_y <= frame_y + 1;						 						 	 
					  end
				 end
             else begin
				   r_current_state <= WRITE_REQUEST;
               frame_x <= frame_x + 1;                             
					pixAddr <= pixAddr + 32'h00000004;						 
				 end
				 				 
			  end // PROCESSDATA Case ends			  			 			  			 
			  
			  WRITE_REQUEST: begin
			    IP2Bus_MstWr_Req <= 1'b1;				 
				 IP2Bus_MstWr_d <= r_pixel_write;
				 IP2Bus_Mst_Addr <= r_write_addr;				 
				 
				 if( Bus2IP_Mst_CmdAck == 1'b1 )
				   begin
					  IP2Bus_MstWr_Req <= 1'b0;
					  r_current_state <= WRITE;					  
				   end
					
				 if( Bus2IP_Mst_Cmplt == 1'b1 )
				   begin
					  r_current_state <= REQUEST;          					  
				   end
			  end
			  
			  WRITE: begin			    
				 IP2Bus_MstWr_d <= r_pixel_write;
				 IP2Bus_Mst_Addr <= r_write_addr;
				 
				 if (Bus2IP_Mst_Cmplt == 1'b1) 
				   begin                 				  
					  r_current_state <= REQUEST;					  
				   end
				 else
					r_current_state <= WRITE;					
			  end
			  
			  CALCULATE: begin			  			      
			    if( r_start == 0 ) begin			      			
               r_done_1 <= 0;
					r_done_2 <= 0;
					r_done_3 <= 0;
					r_done_4 <= 0;
					r_ready_rec_1 <= 0;
					r_ready_rec_2 <= 0;
					r_ready_rec_3 <= 0;
					r_ready_rec_4 <= 0;
					r_start <= 1;
             end					
				 
				 if( r_start == 1 ) begin				   				
				   if( d_ready_1 == 1 ) begin					  					  
					  r_done_1 <= 1;
					  r_ready_rec_1 <= 1;					  
					end
               if( d_ready_2 == 1 ) begin					    					  
					  r_done_2 <= 1;
					  r_ready_rec_2 <= 1;					  
					end
					if( d_ready_3 == 1 ) begin 					  
					  r_done_3 <= 1;
					  r_ready_rec_3 <= 1;					  
					end
               if( d_ready_4 == 1 ) begin					  					  					  
					  r_done_4 <= 1;
					  r_ready_rec_4 <= 1;					  
					end	
				 	
               if ( r_done_1 == 1 && r_done_2 == 1 && r_done_3 == 1 && r_done_4 == 1) begin  
					
					  if (r_player1_point1_count != 0 && (r_player1_point1_x != 0 || r_player1_point1_y != 0)) begin
					    prev_player1_point1_x <= d_quotient_1;
						 prev_player1_point1_y <= d_quotient_2;						   						   
						 
						 if( d_quotient_1 != 0 )
						   slv_reg10 <= 15;
						 else
                     slv_reg10 <= 0;
							
						 slv_reg8 <=  d_quotient_1;
						 slv_reg9 <=  d_quotient_2;																				   					  
					  end
					  else begin						   
					    slv_reg8 <=  prev_player1_point1_x;
						 slv_reg9 <=  prev_player1_point1_y;							
					  end
					
                 slv_reg10 <= r_player1_point1_count;
					  slv_reg11 <= r_player2_point1_count;
					  
					  if ( r_player2_point1_count != 0  && (r_player2_point1_x != 0 || r_player2_point1_y != 0)) begin
						 prev_player2_point1_x <= d_quotient_3;
                   prev_player2_point1_y <= d_quotient_4;														
							
						 slv_reg14 <= d_quotient_3;
						 slv_reg15 <= d_quotient_4;													
					  end
                 else begin
                   slv_reg14 <= prev_player2_point1_x;
						 slv_reg15 <= prev_player2_point1_y;									 
                 end
                 					  
					  r_start <= 0;	 
					  r_current_state <= START_OF_FRAME;						 
					end  // if all 4 points divided					  					 
				 end    // if start is 1
			  end
			  
			  default r_current_state <= INIT; 			  
		  endcase
		end
	end
		  
  // implement slave model register(s)
  always @( posedge Bus2IP_Clk )
    begin: SLAVE_REG_WRITE_PROC
      if ( Bus2IP_Reset == 1 )
        begin
		    r_frame_base_addr <= 0;
			 r_run <= 0;
			 r_rgb_point1_low <=0;
			 r_rgb_point1_high <=0;
			 r_rgb_point2_low <=0;
			 r_rgb_point2_high <=0;
			 r_rgb_point3_low <=0;
			 r_rgb_point3_high <=0;
        end
      else begin		         			
        case ( slv_reg_write_sel )
          24'b100000000000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_frame_base_addr[bit_index] <= Bus2IP_Data[bit_index];
          24'b010000000000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_run[bit_index] <= Bus2IP_Data[bit_index];
          24'b001000000000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_rgb_point1_low[bit_index] <= Bus2IP_Data[bit_index];
          24'b000100000000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_rgb_point1_high[bit_index] <= Bus2IP_Data[bit_index];
          24'b000010000000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_rgb_point2_low[bit_index] <= Bus2IP_Data[bit_index];
          24'b000001000000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_rgb_point2_high[bit_index] <= Bus2IP_Data[bit_index];
          24'b000000100000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_rgb_point3_low[bit_index] <= Bus2IP_Data[bit_index];
          24'b000000010000000000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  r_rgb_point3_high[bit_index] <= Bus2IP_Data[bit_index];          
          default : ;
        endcase
      end  
    end // SLAVE_REG_WRITE_PROC

  // implement slave model register read mux
  always @( slv_reg_read_sel or r_frame_base_addr or r_run or r_rgb_point1_low or r_rgb_point1_high or r_rgb_point2_low or r_rgb_point2_high or r_rgb_point3_low or r_rgb_point3_high or slv_reg8 or slv_reg9 or slv_reg10 or slv_reg11 or slv_reg12 or slv_reg13 or slv_reg14 or slv_reg15 or slv_reg16 or slv_reg17 or slv_reg18 or slv_reg19 or slv_reg_20 or slv_reg_21 or slv_reg_22 or slv_reg_23 )
    begin: SLAVE_REG_READ_PROC

      case ( slv_reg_read_sel )
        24'b100000000000000000000000 : slv_ip2bus_data <= r_frame_base_addr;
        24'b010000000000000000000000 : slv_ip2bus_data <= r_run;
        24'b001000000000000000000000 : slv_ip2bus_data <= r_rgb_point1_low;
        24'b000100000000000000000000 : slv_ip2bus_data <= r_rgb_point1_high;
        24'b000010000000000000000000 : slv_ip2bus_data <= r_rgb_point2_low;
        24'b000001000000000000000000 : slv_ip2bus_data <= r_rgb_point2_high;
        24'b000000100000000000000000 : slv_ip2bus_data <= r_rgb_point3_low;
        24'b000000010000000000000000 : slv_ip2bus_data <= r_rgb_point3_high;
        24'b000000001000000000000000 : slv_ip2bus_data <= slv_reg8;
        24'b000000000100000000000000 : slv_ip2bus_data <= slv_reg9;
        24'b000000000010000000000000 : slv_ip2bus_data <= slv_reg10;
        24'b000000000001000000000000 : slv_ip2bus_data <= slv_reg11;
        24'b000000000000100000000000 : slv_ip2bus_data <= slv_reg12;
        24'b000000000000010000000000 : slv_ip2bus_data <= slv_reg13;
        24'b000000000000001000000000 : slv_ip2bus_data <= slv_reg14;
        24'b000000000000000100000000 : slv_ip2bus_data <= slv_reg15;
        24'b000000000000000010000000 : slv_ip2bus_data <= slv_reg16;
        24'b000000000000000001000000 : slv_ip2bus_data <= slv_reg17;
        24'b000000000000000000100000 : slv_ip2bus_data <= slv_reg18;
        24'b000000000000000000010000 : slv_ip2bus_data <= slv_reg19;
        24'b000000000000000000001000 : slv_ip2bus_data <= slv_reg_20;
        24'b000000000000000000000100 : slv_ip2bus_data <= slv_reg_21;
        24'b000000000000000000000010 : slv_ip2bus_data <= slv_reg_22;
        24'b000000000000000000000001 : slv_ip2bus_data <= slv_reg_23;
        default : slv_ip2bus_data <= 0;
      endcase

    end // SLAVE_REG_READ_PROC

  // ------------------------------------------------------------
  // Example code to drive IP to Bus signals
  // ------------------------------------------------------------

  assign IP2Bus_Data    = slv_ip2bus_data;
  assign IP2Bus_WrAck   = slv_write_ack;
  assign IP2Bus_RdAck   = slv_read_ack;
  assign IP2Bus_Error   = 0;

  // INSTANTIATE THE COLOR SPACE CONVERTER
	divide divide_1
	(
	   .ready      (d_ready_1),
		.quotient   (d_quotient_1),
		.remainder  (),
		.dividend   (d_dividend_1),
		.divider    (d_divider_1),
		.sign       (1'b0),
		.clk        (Bus2IP_Clk),
		.start      (d_start),
		.ready_rec  (d_ready_rec_1)
	);
	
	divide divide_2
	(
	   .ready      (d_ready_2),
		.quotient   (d_quotient_2),
		.remainder  (),
		.dividend   (d_dividend_2),
		.divider    (d_divider_2),
		.sign       (1'b0),
		.clk        (Bus2IP_Clk),
		.start      (d_start),
		.ready_rec  (d_ready_rec_2)
	);
	
	divide divide_3
	(
	   .ready      (d_ready_3),
		.quotient   (d_quotient_3),
		.remainder  (),
		.dividend   (d_dividend_3),
		.divider    (d_divider_3),
		.sign       (1'b0),
		.clk        (Bus2IP_Clk),
		.start      (d_start),
		.ready_rec  (d_ready_rec_3)
	);
	
	divide divide_4
	(
	   .ready      (d_ready_4),
		.quotient   (d_quotient_4),
		.remainder  (),
		.dividend   (d_dividend_4),
		.divider    (d_divider_4),
		.sign       (1'b0),
		.clk        (Bus2IP_Clk),
		.start      (d_start),
		.ready_rec  (d_ready_rec_4)
	);
endmodule

// Divider module
module divide(ready,quotient,remainder,dividend,divider,sign,clk, start, ready_rec);

   input         clk;
	input         start;
	input         ready_rec;
   input         sign;
   input [31:0]  dividend, divider;
   output [31:0] quotient, remainder;
   output        ready;

   reg [31:0]    quotient, quotient_temp;
   reg [63:0]    dividend_copy, divider_copy, diff;
   reg           negative_output;
   
   wire [31:0]   remainder = (!negative_output) ? 
                             dividend_copy[31:0] : 
                             ~dividend_copy[31:0] + 1'b1;

   reg [5:0]     bit; 
   reg           r_start;   
   wire          ready = !bit;

   initial bit = 6'd32;
   initial negative_output = 0;
   initial r_start = 1;

   always @( posedge clk ) 
        
     if( start && r_start ) begin
        r_start = 0;
        bit = 6'd32;
        quotient = 0;
        quotient_temp = 0;
        dividend_copy = (!sign || !dividend[31]) ? 
                        {32'd0,dividend} : 
                        {32'd0,~dividend + 1'b1};
        divider_copy = (!sign || !divider[31]) ? 
                       {1'b0,divider,31'd0} : 
                       {1'b0,~divider + 1'b1,31'd0};

        negative_output = sign &&
                          ((divider[31] && !dividend[31]) 
                        ||(!divider[31] && dividend[31]));
        
     end 
     else if ( bit > 0 ) begin

        diff = dividend_copy - divider_copy;

        quotient_temp = quotient_temp << 1;

        if( !diff[63] ) begin

           dividend_copy = diff;
           quotient_temp[0] = 1'd1;

        end

        quotient = (!negative_output) ? 
                   quotient_temp : 
                   ~quotient_temp + 1'b1;

        divider_copy = divider_copy >> 1;
        bit = bit - 1'b1;

     end
     else if (bit == 0 && ready_rec == 1'b1) begin        
        r_start = 1;
        bit = 6'd32;		  
     end
endmodule
	

