//Author:  Andrew Shorten
//Date:	  April 2009
//Purpose: Demonstrate the functionality of VGA.h

// Video Setup
#include <xbasic_types.h>
#include <xparameters.h>
#include <xiic_l.h>
#include <xuartlite.h>

#include "VGA.h"
#include "List.h"
#include "DrawText.h"
#include "constants.h"
#include "../drivers/position_detector_v1_00_a/src/position_detector.h"

#define TUART_DID		XPAR_UARTLITE_0_DEVICE_ID
#define GPO_REG_OFFSET 0x124
#define DECODER_ADDR 0x20 //Read: 0x41, Write: 0x40
#define SEND_CNT 3
#define RECV_CNT 3
#define GPO_RESETS_OFF 1
#define GPO_RESET_IIC 3
#define GPO_RESET_DECODER 0
#define POSLOC_BASE 0x87000000

//Initialize bouncing box viariables	
const int black = 0x00000000;
const int white  = 0x00FFFFFF;
const int blue = 0x000000FF;
const int cornflowerblue = 0x006495ED;
const int maroon = 0x00800000;
const int firebrick4  = 0x008B1A1A;
const int sgibrightgray = 0x00C5C1AA;
const int wheat2 = 0xEED8AE;
const int lavenderblush = 0x00FFF0F5;
const int buttermilk = 0x00FEF1B5;
const int corn = 0x00FBEC5D;

const int ch=16, cw=12;
	
XUartLite 	TUART_RX_TX;
int to_be_deleted[MAX_SIZE], g1_time_count, g2_time_count, b4_start;
struct Gun g1 = { {0,240-(30/2)}, {2,2}, 1, 0 };
struct Gun g2 = { {640,240-(30/2)}, {2,2}, 2, 0 };

// Initializing IP Core address pointers
volatile Xuint32* frame_base_addr	= (volatile Xuint32*)(POSLOC_BASE + 0x00);
volatile Xuint32* can_run			  	= (volatile Xuint32*)(POSLOC_BASE + 0x04);
volatile Xuint32* rgb_color1_low  	= (volatile Xuint32*)(POSLOC_BASE + 0x08);
volatile Xuint32* rgb_color1_high  	= (volatile Xuint32*)(POSLOC_BASE + 0x0C);
volatile Xuint32* rgb_color2_low  	= (volatile Xuint32*)(POSLOC_BASE + 0x10);
volatile Xuint32* rgb_color2_high  	= (volatile Xuint32*)(POSLOC_BASE + 0x14);
volatile Xuint32* rgb_color3_low  	= (volatile Xuint32*)(POSLOC_BASE + 0x18);
volatile Xuint32* rgb_color3_high  	= (volatile Xuint32*)(POSLOC_BASE + 0x1C);
volatile Xuint32* pos_loc_p1_x1  	= (volatile Xuint32*)(POSLOC_BASE + 0x20);
volatile Xuint32* pos_loc_p1_y1  	= (volatile Xuint32*)(POSLOC_BASE + 0x24);
volatile Xuint32* pos_loc_p1_x2  	= (volatile Xuint32*)(POSLOC_BASE + 0x28);
volatile Xuint32* pos_loc_p1_y2  	= (volatile Xuint32*)(POSLOC_BASE + 0x2C);
volatile Xuint32* pos_loc_p1_count_1  	= (volatile Xuint32*)(POSLOC_BASE + 0x30);
volatile Xuint32* pos_loc_p1_count_2  	= (volatile Xuint32*)(POSLOC_BASE + 0x34);
volatile Xuint32* pos_loc_p2_x1  	= (volatile Xuint32*)(POSLOC_BASE + 0x38);
volatile Xuint32* pos_loc_p2_y1  	= (volatile Xuint32*)(POSLOC_BASE + 0x3C);
volatile Xuint32* pos_loc_p2_x2  	= (volatile Xuint32*)(POSLOC_BASE + 0x40);
volatile Xuint32* pos_loc_p2_y2  	= (volatile Xuint32*)(POSLOC_BASE + 0x44);
volatile Xuint32* pos_loc_p2_count_1  	= (volatile Xuint32*)(POSLOC_BASE + 0x48);
volatile Xuint32* pos_loc_p2_count_2  	= (volatile Xuint32*)(POSLOC_BASE + 0x4C);
volatile Xuint32* frame_pixel       = (volatile Xuint32*)(POSLOC_BASE + 0x50);
volatile Xuint32* debug_1		      = (volatile Xuint32*)(POSLOC_BASE + 0x54);
volatile Xuint32* debug_2		      = (volatile Xuint32*)(POSLOC_BASE + 0x58);
volatile Xuint32* debug_3		      = (volatile Xuint32*)(POSLOC_BASE + 0x5C);

volatile Xuint32* xps_tft_control   = (volatile Xuint32*)(0x85000000);

int point1_r_min, point1_r_max, point1_g_min, point1_g_max, point1_b_min, point1_b_max;
int point2_r_min, point2_r_max, point2_g_min, point2_g_max, point2_b_min, point2_b_max;
int point3_r_min, point3_r_max, point3_g_min, point3_g_max, point3_b_min, point3_b_max;
int can_exit = 0, r, g;
int bytesRecd, i, res = 0, restart_game = 0;	
	
// Struct's for Video Module
struct VideoModule {
  Xuint8 addr;
  Xuint8 config_val;
  Xuint8 actual_val;
};


//Composite Configuration constants
#define DECODER_COMP_CONFIG_CNT 18
struct VideoModule decoder_comp[] = { 
  { 0x00, 0x04, 0 },
  { 0x15, 0x00, 0 },
  { 0x17, 0x41, 0 },
  { 0x27, 0x58, 0 },
  { 0x3a, 0x16, 0 }, 
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x50, 0x20, 0 },
  { 0x52, 0x18, 0 }, 
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xd7, 0xea, 0 },
  { 0xe4, 0x3e, 0 },
  { 0xea, 0x0f, 0 }, 
  { 0x0e, 0x00, 0 } };


void configDecoder(struct VideoModule *decoder, int config_cnt );

//Draws a bouncing box on the supplied screen
void init_position_detector_registers();
int abs_func(int a, int b);
void setMove();
int draw_output(struct Screen* in);
int detect_collision(struct Gun g, int no);
void draw_gun(struct Screen* in, struct Gun g, int no);
void draw_bullet(struct Screen* in, struct Bullet b);
void construct(struct Point gunpoint, struct line gunlines[7], int no);
void construct_b(struct Bullet b, struct line b_lines[4]);
void draw_text(struct Screen *in, struct Point *p, char *str, int length, int color);
void draw_num(struct Screen *in, struct Point *p, int number, int color);
void draw_char(struct Screen *in, struct Point p, char c, int is_digit, int color);
void draw_bar(struct Screen *in, struct Point p, int index, int color);
void draw_score(struct Screen *in);
void draw_time(struct Screen *in);
void draw_line(int X1, int Y1, int X2, int Y2, const int colour, struct Screen *in);
int reverse(int x);
void initialize();

int main()
{   	
   print("\r\nBeginning Setup\r\n");
	configDecoder(decoder_comp, DECODER_COMP_CONFIG_CNT);
	print("\r\nSetup Complete!\r\n\r\n");

   // Configure Detection Point Colours	
	init_position_detector_registers();	
	
   Xuint8 pDataBufferPtr[4];	
	
   /**************************************************************************/
	/* UART                                                                   */
	/**************************************************************************/
	//initialise the UART
	XUartLite_Initialize (&TUART_RX_TX, TUART_DID);
	//self test
	XUartLite_SelfTest(&TUART_RX_TX);
	//Flush all the data in fifo
	XUartLite_ResetFifos(&TUART_RX_TX);
   
	xil_printf("Frame Pixel Begin \n");
   *xps_tft_control = 0x43000000;
	
	// Print frame pixel value read from video-to-ram's video frame in PD
	while(1)
	{
	   i=0;
		
		bytesRecd = XUartLite_Recv(&TUART_RX_TX,  pDataBufferPtr, 4);
		XUartLite_Send(&TUART_RX_TX,  pDataBufferPtr, bytesRecd);
		while(i<bytesRecd)
		{
		    if(pDataBufferPtr[i] == 's')
			     can_exit = 1;    
			 i++;
			 //XUartLite_Send(&TUART_RX_TX,  "Received something", 25);
			 //printf("Received something");
		}
		if(can_exit == 1)
		    break;			 		
		
		int pixel = *frame_pixel, r = (pixel & 0x00FF0000), g = (pixel & 0x0000FF00), b = (pixel & 0x000000FF);
		r = r>>16;
		g = g>>8;
		b = b;
		xil_printf(" R: %d", r);
		xil_printf(" G: %d", g);
		xil_printf(" B: %d", b);
		xil_printf(" Frame Pixel: %d", pixel);		
	}
	xil_printf("Frame Pixel End\n");			
	
	//Declare vga output variables	
	
   while(1)
	{
	
		struct Buffered_screen buf;
		struct Screen * vga1    = (struct Screen *) (0x45000000);
		struct Screen * vga2    = (struct Screen *) (0x46000000);
		struct Screen * temp;
		
		//Initialze vga output variables
		init_buffered_screen(&buf, vga1, vga2, (int *) 0x85000000);			
					
		initialize();
		
		// Wait for an input before playing the game				
		while(1)
		{
			//Get the screen that can be written to
			temp = get_undisplayed(&buf);
			
			//Draw the bouncing box on the screen					
			res = draw_output(temp);				
			if (g1_time_count == 150)
			  break;
			  
			//Swap the buffers
			swap(&buf);
			g1_time_count++;
		}
			
		xil_printf("s \nStarting the game: ");	
		
		g1_time_count = 0;
		g2_time_count = 8;
		b4_start = 0;
		can_exit = 0;
		
		//Play the game
		while(1)
		{
			i=0;
			
			bytesRecd = XUartLite_Recv(&TUART_RX_TX,  pDataBufferPtr, 4);
			XUartLite_Send(&TUART_RX_TX,  pDataBufferPtr, bytesRecd);
			while(i<bytesRecd)
			{
				 xil_printf("Reaching : %c", pDataBufferPtr[i]);
				 if(pDataBufferPtr[i] == 's')
					  can_exit = 1;    
				 i++;
				 //XUartLite_Send(&TUART_RX_TX,  "Received something", 25);
				 //printf("Received something");
			}
			if(can_exit == 1)
				 break;
			
			setMove();
			
			//Get the screen that can be written to
			temp = get_undisplayed(&buf);
			
			//Draw the bouncing box on the screen					
			res = draw_output(temp);		
			if(res)
			{
				 if(res == 1) 
				 { 
					 xil_printf("Collision!!! Player 1 Loses!"); 				 
				 }				 
				 if(res == 2) 
				 { 
					 xil_printf("Collision!!! Player 2 Loses!"); 				 
				 }
				 swap(&buf);
				 break;
			}
			
			//Swap the buffers
			swap(&buf);						
		}
		
		can_exit = 0;
		restart_game = 0;
		
		while(1)
		{
			i=0;
			
			bytesRecd = XUartLite_Recv(&TUART_RX_TX,  pDataBufferPtr, 4);
			XUartLite_Send(&TUART_RX_TX,  pDataBufferPtr, bytesRecd);
			while(i<bytesRecd)
			{
				 if(pDataBufferPtr[i] == 'b')
				 {
					restart_game = 1;
					can_exit = 1; 
				 }
				 else if(pDataBufferPtr[i] == 's')
					can_exit = 1;			 
				 i++;
				 //XUartLite_Send(&TUART_RX_TX,  "Received something", 25);
				 //printf("Received something");
			}
			if(can_exit == 1)
				 break;			 					
		}
		
		if(restart_game == 0)
		{
		  temp = get_undisplayed(&buf);
		  break;
		}
	}
	
	xil_printf("Reaching");
	return 0;
}

void initialize()
{
  Init_b();
  set_alphanum();	
	
  g1.lives = 10; 
  g2.lives = 10;
  g1.is_triggered = 0;
  g2.is_triggered = 0;    
  g1_time_count = 0;
  g2_time_count = 0;
  
  b4_start = 1;   		
  can_exit = 0;
  bytesRecd = 0;
  res = 0;	
}

void init_position_detector_registers()
{
  // Point 3 : Red R=225,G=0, B=0
  point3_r_max = 255; point3_r_min = 190;
  point3_g_max = 70;  point3_g_min = 0;
  point3_b_max = 70;  point3_b_min = 0;
  
//  point1_r_max = 90;  point1_r_min = 0;
//  point1_g_max = 90;  point1_g_min = 0;
//  point1_b_max = 255;   point1_b_min = 170;
  
  // Point 1 : Blue R=0,G=0,B=225 
  point1_r_max = 90;  point1_r_min = 0;
  point1_g_max = 90;  point1_g_min = 0;
  point1_b_max = 190;   point1_b_min = 100;
  
  // Point 2 : Yellow R=236, G=236, B=0
  point2_r_max = 255;  point2_r_min = 170;
  point2_g_max = 255;  point2_g_min = 150;
  point2_b_max = 70;  point2_b_min = 0;
  
  // Set Color Ranges
  // volatile Xuint32* rgb_color1_low  	= (volatile Xuint32*)(POSLOC_BASE + 0x08);
  *rgb_color1_low   = (point1_r_min << 24) | (point1_g_min << 16) | (point1_b_min <<  8);
  *rgb_color1_high  = (point1_r_max << 24) | (point1_g_max << 16) | (point1_b_max <<  8);
  *rgb_color2_low   = (point2_r_min << 24) | (point2_g_min << 16) | (point2_b_min <<  8);
  *rgb_color2_high  = (point2_r_max << 24) | (point2_g_max << 16) | (point2_b_max <<  8);
  *rgb_color3_low   = (point3_r_min << 24) | (point3_g_min << 16) | (point3_b_min <<  8);
  *rgb_color3_high  = (point3_r_max << 24) | (point3_g_max << 16) | (point3_b_max <<  8);
  
  *can_run = 0xF000000F;
  // Set Gun Initial Positions  
  //  *pos_loc_p1_x1 = 0;
  //  *pos_loc_p1_y1 = 240-(30/2); 
  //  *pos_loc_p2_x1 = 640; 
  //  *pos_loc_p2_y1 = 240-(30/2); 
}

void configDecoder(struct VideoModule *decoder, int config_cnt ) 
{
  Xuint16 send_cnt, i;
  Xuint8 send_data[2] = {0};
  Xuint8 success = 1;
  send_cnt = 2;
  xil_printf("  Configuring Decoder...\t");
  //read in configuration constants and send them to the VDEC one by one
  for( i = 0; i < config_cnt; i++ )
   {

    send_data[0] = decoder[i].addr;

    send_data[1] = decoder[i].config_val;

    send_cnt = XIic_Send(XPAR_IIC_BASEADDR, DECODER_ADDR, send_data, 2,XIIC_STOP );

    if( send_cnt != 2 ) 
	 {
      xil_printf("Error writing to address %02x\r\n", decoder[i].addr);
      success = 0;
      break;
     }
   }

  if( success )
    xil_printf("SUCCESS!\r\n");

}

int abs_func(int a, int b)
{
   if(a>b) return a-b;
	else return b-a;
}

//Draws a bouncing box on the supplied screen
int draw_output(struct Screen* in)
{
	//Temporary variables
	int i, j, cx, cy;
	int result;	
	struct Bullet b;
	const int box_size = 30;	
	
	//Clear screen
	clear_screen(in, corn);	
	
	draw_score(in);
	
	//draw partition
	for(i = 300; i < 340; i++)
	{
		for(j = 0; j < 480; j++)
		{
			write_pixel(in, cornflowerblue, i, j);
		}
	}		   		
	
	g1.start_loc.x = (int)*pos_loc_p1_x1; 
	g1.start_loc.y = (int)*pos_loc_p1_y1; 
	g2.start_loc.x = (int)*pos_loc_p2_x1; 
	g2.start_loc.y = (int)*pos_loc_p2_y1;			 			   		
	
	g1.start_loc.x = 640 - g1.start_loc.x;
	g2.start_loc.x = 640 - g2.start_loc.x;
	
	//update bullet locations
   for(i = 0; i < size_bullets; i++)
	{
	    b = get(i);
		 struct Point p = { b.start_loc.x + b.velocity.x, b.start_loc.y + b.velocity.y };
	    set(i, p, b.velocity);
		 if((b.velocity.x > 0 && b.start_loc.x > 640-9) || (b.velocity.x < 0 && b.start_loc.x < 0+9))
		 {
		     delete_b(i);
			  continue;
	    }		 	    
	}
	
	if(g1.is_triggered) 
	{
	    struct Point p = { g1.start_loc.x + box_size, g1.start_loc.y }, s = { 16, 0};			 
	    add_bullet(p, s);		 
   }
	if(g2.is_triggered) 
	{
	    struct Point p = { g2.start_loc.x - box_size, g2.start_loc.y }, s = { -16, 0};			 
	    add_bullet(p, s);		 	    
   }
		
	// Detect Collision
   for(i=0; i<size_bullets; i++)   to_be_deleted[i]=0;
	
	result = 1;
	result = detect_collision(g1, 1);	
	if(!result) 
	{ 
	   g1.lives--;      
		if(g1.lives == 0)
      {
         struct Point p1 = { 220, 210 }, p2 = { 200, 250 };
	      draw_text(in, &p1, "PLAYER TWO WINS", 15, firebrick4);
         draw_text(in, &p2, "PRESS B TO RESTART", 18, firebrick4);			
		   return 1; 
		}
	}
	
	result = 1;		
	result = detect_collision(g2, 2);
	if(!result) 
	{ 
	   g2.lives--; 		
		if(g2.lives == 0) 
		{
		   struct Point p1 = { 220, 210 }, p2 = { 200, 250 };
	      draw_text(in, &p1, "PLAYER ONE WINS", 15, firebrick4);
			draw_text(in, &p2, "PRESS B TO RESTART", 18, firebrick4);
		   return 2; 
		}
	}
			
	// Draw Bullets
	for(i = 0; i < size_bullets; i++)
	    draw_bullet(in, get(i));
	
	for(i = 0; i < size_bullets; i++)
		if(to_be_deleted[i])
			delete_b(i);

	draw_gun(in, g1, 1);
	draw_gun(in, g2, 2);

   if(b4_start)
	{
	  draw_time(in);     	 
	}
	else
	{
	  if(g1.start_loc.x > 300 && g1.start_loc.x < 340)
	  {
	    g1.lives--;      
		 if(g1.lives == 0)
       {
		   struct Point p1 = { 220, 210 }, p2 = { 200, 250 };	               			
	      draw_text(in, &p1, "PLAYER ONE LOSES", 16, firebrick4);
         draw_text(in, &p2, "PRESS B TO RESTART", 18, firebrick4);			
		   return 1;
		 }	    
	  }
	  if(g2.start_loc.x > 300 && g2.start_loc.x < 340)
	  {
	    g2.lives--;      
		 if(g2.lives == 0)
       {
         struct Point p1 = { 220, 210 }, p2 = { 200, 250 };	
	      draw_text(in, &p1, "PLAYER TWO LOSES", 16, firebrick4);
         draw_text(in, &p2, "PRESS B TO RESTART", 18, firebrick4);			
		   return 1;
		 }
	  }
	}
	
	return 0;
}

void draw_gun(struct Screen* in, struct Gun g, int no)
{
   int i,j,cx,cy,px,py;
	int x = g.start_loc.x, y = g.start_loc.y;	
   const int box_size = 30;	

   //draw Gun pointer
	for(i = 0; i < (box_size/5); i++)
	{
		for(j = 0; j < box_size; j++)
		{
		   if(no==1) { px = x + i; } else { px = x - i; }
			write_pixel(in, black, px, y + j);
		}
	}
	
	//draw Gun barrel		
	for(i = 0; i < box_size; i++)
	{
		for(j = 0; j < (box_size/5); j++)
		{
		   if(no==1) { px=x+i; } else { px=x-i; }
			write_pixel(in, black, px, y + j);
		}
	}

   if(no==1) cx = x+(box_size/5)-1;
	else cx = x-(box_size/5)+1;
   cy = y+(box_size/5)-1;
	
	// draw trigger
	for(i = cx,j = cy+(2*box_size/5)-1; j > cy; j--)
	{		
			write_pixel(in, black, i, j);
         if(no==1) i++;    else i--;
	} 
}

void draw_bullet(struct Screen* in, struct Bullet b)
{	 
    int i,j, bx;
	 
    for(i=0; i<9; i++)
	 {
	    for(j=0; j<3; j++)
		 {
		    if(b.velocity.x >= 0) bx = b.start_loc.x+i; else bx = b.start_loc.x-i;
			 write_pixel(in, black, bx, b.start_loc.y + j);
		 }
	 }
}

void setMove()
{  
  if(g1.is_triggered == 1)
   g1.is_triggered = 0; 
  if(g2.is_triggered == 1)	
	g2.is_triggered = 0;
	
  if(g1_time_count == 16)
  {
    g1.is_triggered = 1;
	 g1_time_count = 0;
  }
  if(g2_time_count == 16)
  {
    g2.is_triggered = 1;
	 g2_time_count = 0;
  }

  g1_time_count++;
  g2_time_count++;  
}

int detect_collision(struct Gun g, int no)
{   
   struct Point gunpoint = { g.start_loc.x, g.start_loc.y } ;   	
   struct line gun_lines[7];
	construct(gunpoint, gun_lines, no);		
	const int box_size = 30;
	int k,i,j;
			
	for(k=0; k<size_bullets; k++)
	{
      int no_of_intrscs = 0;		
		struct line b_lines[4];
		
		struct Bullet b = get(k);	
		if(b.velocity.x > 0 && no == 1)   continue;
		if(b.velocity.x < 0 && no == 2)   continue;
		
		construct_b(b, b_lines);
      		
		for(i=0; i<7; i++)
	   {      
			for(j=0; j<4; j++)
			   if(intersect(gun_lines[i], b_lines[j]))
            {				
				    no_of_intrscs++;						    									 
			   }
		}
		if(no_of_intrscs > 0)
		{
		   to_be_deleted[k]=1;
		   //delete_b(k);		  
		   xil_printf("Intersecting : %d ", no_of_intrscs);
		   return 0;
		}
		else
		{		  
		   int ycheck = b.start_loc.y - gun_lines[0].p2.y;
			if(ycheck < 0) continue;
						
		   if(no == 1 && b.start_loc.x < gun_lines[0].p2.x && ycheck <= (box_size/5))
			{
			   to_be_deleted[k]=1;
			   //delete_b(k);			   
			   // xil_printf("Inside gun 1 !");
			   return 0;
			}
			if(no == 2 && b.start_loc.x > gun_lines[0].p2.x && ycheck <= (box_size/5))
			{
			   to_be_deleted[k]=1;
			   //delete_b(k);			   
			   // xil_printf("Inside gun 2 !");
			   return 0;	
			}
		}
	}	
	
	return 1;
}

void construct(struct Point gunpoint, struct line gunlines[7], int no)
{	   
   const int box_size = 30;
   if(no == 1)
	{
	   struct line gunlines0 = { {gunpoint.x, gunpoint.y}, {gunpoint.x+box_size-1,gunpoint.y} };
		struct line gunlines1 = { {gunpoint.x+box_size-1, gunpoint.y}, {gunpoint.x+box_size-1,gunpoint.y+(box_size/5)-1} };
		struct line gunlines2 = { {gunpoint.x+3*(box_size/5)-2, gunpoint.y+(box_size/5)-1}, {gunpoint.x+box_size-1,gunpoint.y+(box_size/5)-1} };
		struct line gunlines3 = { {gunpoint.x+(box_size/5)-1, gunpoint.y+3*(box_size/5)-2}, {gunpoint.x+3*(box_size/5)-2, gunpoint.y+(box_size/5)-1} };
		struct line gunlines4 = { {gunpoint.x+(box_size/5)-1, gunpoint.y+(box_size/5)-1}, {gunpoint.x+(box_size/5)-1, gunpoint.y+box_size-1} };
		struct line gunlines5 = { {gunpoint.x, gunpoint.y+box_size-1}, {gunpoint.x+(box_size/5)-1, gunpoint.y+box_size-1} };
		struct line gunlines6 = { {gunpoint.x, gunpoint.y}, {gunpoint.x, gunpoint.y+box_size-1} };
		
		gunlines[0] = gunlines0;
		gunlines[1] = gunlines1;
		gunlines[2] = gunlines2;
		gunlines[3] = gunlines3;
		gunlines[4] = gunlines4;
		gunlines[5] = gunlines5;
		gunlines[6] = gunlines6;
	}
	if(no == 2)
	{
	   struct line gunlines0 = { {gunpoint.x, gunpoint.y}, {gunpoint.x-box_size+1,gunpoint.y} };
		struct line gunlines1 = { {gunpoint.x-box_size+1, gunpoint.y}, {gunpoint.x-box_size+1,gunpoint.y+(box_size/5)-1} };
		struct line gunlines2 = { {gunpoint.x-3*(box_size/5)+2, gunpoint.y+(box_size/5)-1}, {gunpoint.x-box_size+1,gunpoint.y+(box_size/5)-1} };
		struct line gunlines3 = { {gunpoint.x-(box_size/5)+1, gunpoint.y+3*(box_size/5)-2}, {gunpoint.x-3*(box_size/5)+2, gunpoint.y+(box_size/5)-1} };
		struct line gunlines4 = { {gunpoint.x-(box_size/5)+1, gunpoint.y+(box_size/5)-1}, {gunpoint.x-(box_size/5)+1, gunpoint.y+box_size-1} };
		struct line gunlines5 = { {gunpoint.x, gunpoint.y+box_size-1}, {gunpoint.x-(box_size/5)+1, gunpoint.y+box_size-1} };
		struct line gunlines6 = { {gunpoint.x, gunpoint.y}, {gunpoint.x, gunpoint.y+box_size-1} };
		
		gunlines[0] = gunlines0;
		gunlines[1] = gunlines1;
		gunlines[2] = gunlines2;
		gunlines[3] = gunlines3;
		gunlines[4] = gunlines4;
		gunlines[5] = gunlines5;
		gunlines[6] = gunlines6;
	}
}

void construct_b(struct Bullet b, struct line b_lines[4])
{
   int bh=3, bw=9;
   if(b.velocity.x>0)
	{
	   struct line b_lines0 = { {b.start_loc.x, b.start_loc.y}, {b.start_loc.x+b.velocity.x-1, b.start_loc.y} };
		struct line b_lines1 = { {b.start_loc.x+b.velocity.x-1, b.start_loc.y}, {b.start_loc.x+b.velocity.x-1, b.start_loc.y+bh-1} };
		struct line b_lines2 = { {b.start_loc.x, b.start_loc.y+bh-1}, {b.start_loc.x+b.velocity.x-1, b.start_loc.y+bh-1} };
		struct line b_lines3 = { {b.start_loc.x, b.start_loc.y}, {b.start_loc.x, b.start_loc.y+bh-1} };
		
		b_lines[0] = b_lines0;
		b_lines[1] = b_lines1;
		b_lines[2] = b_lines2;
		b_lines[3] = b_lines3;
	}
	if(b.velocity.x<0)
	{
	   struct line b_lines0 = { {b.start_loc.x, b.start_loc.y}, {b.start_loc.x-b.velocity.x+1, b.start_loc.y} };
		struct line b_lines1 = { {b.start_loc.x-b.velocity.x+1, b.start_loc.y}, {b.start_loc.x-b.velocity.x+1, b.start_loc.y+bh-1} };
		struct line b_lines2 = { {b.start_loc.x, b.start_loc.y+bh-1}, {b.start_loc.x-b.velocity.x+1, b.start_loc.y+bh-1} };
		struct line b_lines3 = { {b.start_loc.x, b.start_loc.y}, {b.start_loc.x, b.start_loc.y+bh-1} };
		
		b_lines[0] = b_lines0;
		b_lines[1] = b_lines1;
		b_lines[2] = b_lines2;
		b_lines[3] = b_lines3;
	}
}

void draw_bar(struct Screen *in, struct Point p, int index, int color)
{
   int i=0, j;
   switch(index)
   {
	   case 0:
		    i=p.x; j=p.y;
		    for(i=p.x;i<p.x+cw; i++)
			     write_pixel(in, color, i, j);
		    break;
	   case 1:
		    i=p.x+cw; j=p.y;
		    for(;j<p.y+(ch/2); j++)
			     write_pixel(in, color, i, j);
		    break;
		case 2:
		    i=p.x+cw; j=p.y+(ch/2);
		    for(;j<p.y+ch; j++)
			     write_pixel(in, color, i, j);
		    break;
		case 3:
		    i=p.x; j=p.y+ch;
		    for(;i<p.x+cw; i++)
			     write_pixel(in, color, i, j);
		    break;
		case 4:
		    i=p.x; j=p.y+(ch/2);
		    for(;j<p.y+ch; j++)
			     write_pixel(in, color, i, j);
		    break;
		case 5:
		    i=p.x; j=p.y;
		    for(;j<p.y+(ch/2); j++)
			     write_pixel(in, color, i, j);
		    break;
		case 6:
		    i=p.x; j=p.y+(ch/2);
		    for(;i<p.x+(cw/2); i++)
			     write_pixel(in, color, i, j);
		    break;
		case 7:
		    i=p.x+(cw/2); j=p.y+(ch/2);
		    for(;i<p.x+cw; i++)
			     write_pixel(in, color, i, j);
		    break;
		case 8:
		    draw_line(p.x, p.y, p.x+(cw/2), p.y+(ch/2), color, in);		    
		    break;
		case 9:		   
		    draw_line(p.x+(cw/2), p.y+(ch/2), p.x+cw, p.y, color, in);
		    break;
		case 10:
		    draw_line(p.x, p.y+ch, p.x+(cw/2), p.y+(ch/2), color, in);
		    break;
		case 11:		    		    
			 draw_line(p.x+(cw/2), p.y+(ch/2), p.x+cw, p.y+ch, color, in);
		    break;
	   case 12:		    		    
			 draw_line(p.x+(cw/2), p.y, p.x+(cw/2), p.y+ch, color, in);
		    break;
	   default:
		    break;
	}
}

void draw_char(struct Screen *in, struct Point p, char c, int is_digit, int color)
{
   if(c == ' ')
	  return;
	  
   int k=0, index = c-'A';
   if(!is_digit)
	{
	   for(k=0; k<13; k++)
		   if(alphabets[index].st[k]==1)
			   draw_bar(in, p, k, color);
	}
	else if(is_digit)
	{
	   index = c - '0';
	   for(k=0; k<13; k++)
		   if(numbers[index].st[k]==1)
			   draw_bar(in, p, k, color);
	}
}

void draw_text(struct Screen *in, struct Point *p, char *str, int length, int color)
{
   int k=0;
   for(k=0; k<length; k++)
	{
	   draw_char(in, *p, str[k], 0, color);
		p->x = p->x + cw + 2;
	}
}

void draw_num(struct Screen *in, struct Point *p, int number, int color)
{
   int digits[10], counter=0, i=0;
	   
	while(number!=0)
	{
	   digits[counter] = number%10;		
		number/=10;		
		counter++;
	}
	
	for(i=counter-1; i>=0; i--)
	{
     draw_char(in, *p, '0'+digits[i], 1, color);
	  p->x = p->x + cw+2;
	}
}

void draw_score(struct Screen *in)
{   
   struct Point p1 = { 20, 20 };	
	struct Point p2 = { 420, 20 };
	int l1 = g1.lives, l2=g2.lives;
	
   draw_text(in, &p1, "PLAYER ONE ", 12, maroon);
	draw_num(in, &p1, l1, blue);	
	
	draw_text(in, &p2, "PLAYER TWO ", 12, maroon);
	draw_num(in, &p2, l2, blue);	
}

void draw_time(struct Screen *in)
{
  struct Point p1 = { 230, 220 };		  
  int l1=0;

  if(g1_time_count < 30)
	 l1 = 5;
  else if(g1_time_count < 60)
	 l1 = 4;
  else if(g1_time_count < 90)
	 l1 = 3;
  else if(g1_time_count < 120)
	 l1 = 2;
  else if(g1_time_count < 150)
	 l1 = 1;
	
  if(l1 == 3)
  {
	 struct Point p2 = { 304, 250 };
	 draw_text(in, &p2, "GET", 3, black);
  }
  if(l1 == 2)
  {
	 struct Point p2 = { 304, 250 };
	 draw_text(in, &p2, "SET", 3, black);
  }
  if(l1 == 1)
  {
	 struct Point p2 = { 318, 250 };
	 draw_text(in, &p2, "GO", 2, black);
  }
  
  draw_text(in, &p1, "TIME TO START ", 14, black);
  draw_num(in, &p1, l1, blue);  
}

void draw_line(int X1, int Y1, int X2, int Y2, const int colour, struct Screen *in)
{
	//Temporary variables
	int steep = 0;
	int dx, dy;
	unsigned int x, y;
	unsigned int temp;
	unsigned int Y;
	int error = 0;
	float deltaerr = 0.0;
	int ystep = 0;
	unsigned int i;
	unsigned int end;
	int p;
	//const int paddle1 = 0xFFFFFFFF; // White
//	const int paddle2 = 0x00FF00FF; // Random colour
	
	if (Y1 > Y2)
		dy = Y1 - Y2;
	else
		dy = Y2 - Y1;
	if (X1 > X2)
		dx = X1 - X2;
	else
		dx = X2 - X1;
		
	if (dy > dx){ //steep
		steep = 1;
		temp = X1;
		X1 = Y1;
		Y1 = temp;
		
		temp = X2;
		X2 = Y2;
		Y2 = temp;
	}
	
	if (X1 > X2){ //make sure X2 is the higher value
		temp = X1;
		X1 = X2;
		X2 = temp;
		
		temp = Y1;
		Y1 = Y2;
		Y2 = temp;
	}
	
	dx = X2 - X1;
	if (Y1 > Y2)
		dy = Y1 - Y2;
	else
		dy = Y2 - Y1;
	
	error = dx/2;
//	y = Y1;
//	deltaerr = dy/dx;
	if (Y1 < Y2)
		ystep = 1;
	else
		ystep = -1;
		
	Y = Y1;
	
	for(i = X1; i < X2; i = i+1) {
		if (steep == 1) {
			write_pixel(in, colour, Y, i);
		}
			
		else {
			write_pixel(in, colour, i, Y);
		}
		error = error - dy;
		if (error < 0 ){
			Y = Y + ystep;
			error = error + dx;
		}
	}
	
}

int reverse(int x) {
 xil_printf(" %d ", x);
 int h = 0;
 int i = 0;

 for(h = i = 0; i < 32; i++) {
  h = (h << 1) + (x & 1); 
  x >>= 1; 
 }

 return h;
}
